import classNames from "classnames";
import Link from "next/link";
import {useRouter} from "next/router";
import React, {useState, useMemo, useEffect} from "react";
import {
    ArticleIcon,
    CollapsIcon,
    LogoIcon,
    LogoutIcon,
    SearchIcon,
    HomeIcon,
    CreateIcon
} from "./icons";

const Sidebar = () => {
    const [toggleCollapse, setToggleCollapse] = useState(false);
    const [isCollapsible, setIsCollapsible] = useState(false);
    const [menuItems, setMenuItems] = useState([
        {id: 1, label: "Home", icon: HomeIcon, link: "/"},
        {id: 2, label: "Search", icon: SearchIcon, link: "/search"},
        {id: 3, label: "Create document", icon: CreateIcon, link: "/create-document"}
    ]);
    const [token, setToken] = useState("");

    useEffect(() => {
        setToken(() => localStorage.getItem("token"))
    }, [])

    useEffect(() => {
        if (token) {
            const fetchData = async () => {
                const res = await fetch("http://localhost:8000/api/documents", {
                    method: "GET",
                    headers: {
                        'Authorization': `Bearer ${token}`
                    }
                })
                let json = await res.json();
                let list = [
                    {id: 1, label: "Home", icon: HomeIcon, link: "/"},
                    {id: 2, label: "Search", icon: SearchIcon, link: "/search"},
                    {id: 3, label: "Create document", icon: CreateIcon, link: "/create-document"}
                ]
                json.forEach((doc) => {
                    list.push({
                        id: doc["id"],
                        label: doc["name"],
                        icon: ArticleIcon,
                        link: `/documents/${doc["id"]}`
                    })
                })

                setMenuItems(() => list)
            }

            fetchData()
        }
    }, [token])

    const router = useRouter();

    let activeMenu = useMemo(
        () => menuItems.find((menu) => {
            if (!router.query.id || router.query.id === 'undefined') {
                return menu.link === router.pathname
            } else if (!(menu.id === 1 || menu.id === 2 || menu.id === 3)) {
                return menu.id.toString() === router.query.id
            }
            return false
        }),
        [menuItems, router.query.id, router.pathname]
    );

    const wrapperClasses = classNames(
        "h-screen px-4 pt-8 pb-4 bg-light flex justify-between flex-col",
        {
            ["w-80"]: !toggleCollapse,
            ["w-20"]: toggleCollapse,
        }
    );

    const collapseIconClasses = classNames(
        "p-4 rounded bg-light-lighter absolute right-0",
        {
            "rotate-180": toggleCollapse,
        }
    );

    const getNavItemClasses = (menu) => {
        if (!activeMenu || !menu || menu === "undefined") {
            if (!router.query || !router.query.id) {
                return classNames("flex items-center cursor-pointer hover:bg-light-lighter rounded w-full overflow-hidden whitespace-nowrap");
            } else {
                return classNames(
                    "flex items-center cursor-pointer hover:bg-light-lighter rounded w-full overflow-hidden whitespace-nowrap",
                    {
                        ["bg-light-lighter"]: router.query.id === menu.id,
                    }
                );
            }
        }
        return classNames(
            "flex items-center cursor-pointer hover:bg-light-lighter rounded w-full overflow-hidden whitespace-nowrap",
            {
                ["bg-light-lighter"]: activeMenu.id === menu.id,
            }
        );
    };

    const onMouseOver = () => {
        setIsCollapsible(!isCollapsible);
    };

    const handleSidebarToggle = () => {
        setToggleCollapse(!toggleCollapse);
    };

    const handleSubmit = async (event) => {
        event.preventDefault()

        const res = await fetch("http://localhost:8000/api/token", {
            method: "DELETE",
            headers: {
                'Authorization': `Bearer ${token}`
            }
        });

        localStorage.removeItem("token")
        router.push("/")
    }

    return (
        <div
            className={wrapperClasses}
            onMouseEnter={onMouseOver}
            onMouseLeave={onMouseOver}
            style={{transition: "width 300ms cubic-bezier(0.2, 0, 0, 1) 0s"}}
        >
            <div className="flex flex-col">
                <div className="flex items-center justify-between relative">
                    <div className="flex items-center pl-1 gap-4">
                        <LogoIcon/>
                        <span
                            className={classNames("mt-2 text-lg font-medium text-text", {
                                hidden: toggleCollapse,
                            })}
                        >
              BSition
            </span>
                    </div>
                    {isCollapsible && (
                        <button
                            className={collapseIconClasses}
                            onClick={handleSidebarToggle}
                        >
                            <CollapsIcon/>
                        </button>
                    )}

                </div>

                <div className="flex flex-col items-start mt-24">
                    {menuItems.map(({icon: Icon, ...menu}) => {
                        const classes = getNavItemClasses(menu);
                        return (
                            <div className={classes} key={menu.id}>
                                <Link href={menu.link}>
                                    <a className="flex py-4 px-3 items-center w-full h-full">
                                        <div style={{width: "2.5rem"}}>
                                            <Icon/>
                                        </div>
                                        {!toggleCollapse && (
                                            <span
                                                className={classNames(
                                                    "text-md font-medium text-text-light"
                                                )}
                                            >
                                                {menu.label}
                                            </span>
                                        )}
                                    </a>
                                </Link>
                            </div>
                        );
                    })}
                </div>
            </div>

            <div className={`${getNavItemClasses({})} px-3 py-4`}>
                <div style={{width: "2.5rem"}}>
                    <LogoutIcon/>
                </div>
                {!toggleCollapse && (
                    <form onSubmit={handleSubmit}>
                        <button name="Logout" value="logout">
                            <span className={classNames("text-md font-medium text-text-light")}>
                                Logout
                            </span>
                        </button>
                    </form>
                )}
            </div>
        </div>
    );
};

export default Sidebar;
